package platform_user

import (
	"encoding/json"
	"fmt"
	"github.com/pkg/errors"
	"io"
	"io/ioutil"
	"live-service/app/user_center/rpc/internal/svc"
	"net/http"
	"net/url"
	"strconv"
	"time"
)

type MyProxy struct {
	Id         int64  `json:"id"`
	Ip         string `json:"ip"`
	Port       string `json:"port"`
	SchemeType int    `json:"scheme_type"`
	Content    string `json:"content"`
}

func RetrieveBilibili(pUid string) (*PlatformUser, error) {
	proxy := func(_ *http.Request) (*url.URL, error) {
		httpResponse, err := http.Get(fmt.Sprintf("http://127.0.0.1:9999/sql?query=%s",
			url.QueryEscape("SELECT * FROM proxy ORDER BY RAND() limit 1")))
		if err != nil {
			return nil, err
		}
		defer httpResponse.Body.Close()

		all, err := io.ReadAll(httpResponse.Body)
		if err != nil {
			return nil, err
		}
		var respStruct struct {
			Error   string    `json:"error"`
			Message []MyProxy `json:"message"`
		}
		err = json.Unmarshal(all, &respStruct)
		if err != nil {
			return nil, err
		}
		u := fmt.Sprintf("http://%s", respStruct.Message[0].Content)
		return url.Parse(u)
	}

	transport := &http.Transport{Proxy: proxy}
	client := &http.Client{Transport: transport}

	httpResponse, err := client.Get(fmt.Sprintf("http://api.bilibili.com/x/space/acc/info?mid=%s", pUid))
	if err != nil {
		return nil, errors.Wrap(err, "网络错误")
	}
	all, err := io.ReadAll(httpResponse.Body)
	if err != nil {
		return nil, errors.Wrap(err, "读取接口数据错误 IO/Error")
	}
	var resp struct {
		Code    int64                  `json:"code"`
		Message string                 `json:"message"`
		Ttl     int64                  `json:"ttl"` // 未知意义
		Data    map[string]interface{} `json:"data,omitempty"`
	}
	err = json.Unmarshal(all, &resp)
	if err != nil {
		return nil, errors.Wrap(err, "读取接口数据错误 Unmarshal/Error")
	}
	if resp.Code == 0 && resp.Data != nil {
		// 成功
		data, err := json.Marshal(resp.Data)
		if err != nil {
			return nil, errors.Wrap(err, "读取接口数据错误 Marshal/Data/Error")
		}
		return &PlatformUser{
			PUid:    strconv.FormatInt(int64(resp.Data["mid"].(float64)), 10),
			PName:   resp.Data["name"].(string),
			PAvatar: resp.Data["face"].(string),
			PInfo:   string(data),
		}, nil
	} else {
		return nil, errors.New(resp.Message)
	}
}

type (
	topList struct {
		Code int `json:"code"`
		Data struct {
			Info struct {
				Num  int `json:"num"`
				Page int `json:"page"`
				Now  int `json:"now"`
			} `json:"info"`
			List []guardItem `json:"list"`
			Top3 []guardItem `json:"top3"`
		} `json:"data"`
	}
	/**
	{
		"uid": 11309307,
		"ruid": 14890801,
		"rank": 4,
		"username": "288号花店店长",
		"face": "http://i0.hdslb.com/bfs/face/b86499c02ed5b02ced46ed82813251bdf61aae60.jpg",
		"is_alive": 0,
		"guard_level": 2,
		"guard_sub_level": 0,
		"medal_info": {
			"medal_name": "耐久",
			"medal_level": 30,
			"medal_color_start": 2951253,
			"medal_color_end": 10329087,
			"medal_color_border": 16771156
		}
	},
	*/
	guardItem struct {
		Uid    int64  `json:"uid"`
		Avatar string `json:"face"`
		Level  int    `json:"guard_level"`
	}
)

func NobilityBilibili(svcCtx *svc.ServiceContext) ([]NobilityData, error) {
	cfg := svcCtx.Config.UserRetriever.Bilibili
	if cfg.Mid == 0 {
		if mId, err := fetchMid(cfg.RoomShortInfoApi, cfg.RoomId); err != nil {
			return nil, err
		} else {
			cfg.Mid = mId
		}
	}
	if cfg.Mid == 0 {
		return nil, errors.New("获取UP主ID失败，无法获取舰长信息")
	}
	// 获取第一页
	tl, err := getNobilityByPage(cfg.TopListApi, cfg.RoomId, cfg.Mid, 1)
	if err != nil {
		return nil, err
	}
	totalNum := tl.Data.Info.Num
	totalPage := tl.Data.Info.Page
	result := make([]NobilityData, 0, totalNum)

	appendArrFn := func(items []guardItem) {
		for _, item := range items {
			result = append(result, NobilityData{
				PUid:    strconv.FormatInt(item.Uid, 10),
				PAvatar: item.Avatar,
				Level:   item.Level,
			})
		}
	}

	// 数据小于1页
	if totalPage <= 1 {
		// num > 3, list 为准, 否则取top3
		if totalNum <= 3 {
			appendArrFn(tl.Data.Top3)
		} else {
			appendArrFn(tl.Data.Top3)
		}
	} else {
		// 数据需要翻页 2 -> totalPage
		for i := 2; i <= totalPage; i++ {
			tl, err = getNobilityByPage(cfg.TopListApi, cfg.RoomId, cfg.Mid, i)
			if err != nil {
				continue
			}
			appendArrFn(tl.Data.List)
			// 睡眠50ms,暂缓b站封ip
			time.Sleep(50 * time.Millisecond)
		}
	}
	return result, nil
}

func fetchMid(api string, roomId int64) (mId int64, err error) {
	// 获取mid
	httpResp, err := http.Get(fmt.Sprintf("%s?id=%d", api, roomId))
	if err != nil {
		return
	}
	res, err := ioutil.ReadAll(httpResp.Body)
	if err != nil {
		return
	}
	var roomInfo struct {
		Code    int    `json:"code"`
		Msg     string `json:"msg"`
		Message string `json:"message"`
		Data    struct {
			RoomId int64 `json:"room_id"`
			Uid    int64 `json:"uid"`
		} `json:"data"`
	}
	err = json.Unmarshal(res, &roomInfo)
	if err != nil {
		return
	}
	mId = roomInfo.Data.Uid
	return
}

func getNobilityByPage(api string, roomId, mId int64, page int) (*topList, error) {
	// 获取舰长列表
	TopListApiFormat := api + "?roomid=%d&ruid=%d&page=%d"
	httpResp, err := http.Get(fmt.Sprintf(TopListApiFormat, roomId, mId, page))
	if err != nil {
		return nil, err
	}
	res, err := ioutil.ReadAll(httpResp.Body)
	if err != nil {
		return nil, err
	}
	var tl topList
	err = json.Unmarshal(res, &tl)
	if err != nil {
		return nil, err
	}
	return &tl, nil
}
