package statistics

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/gorm-zero/gormx"
	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
)

type StatPvpReportLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewStatPvpReportLogic(ctx context.Context, svcCtx *svc.ServiceContext) *StatPvpReportLogic {
	return &StatPvpReportLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *StatPvpReportLogic) StatPvpReport(in *pb.StatPvPReportReq) (*pb.StatPvPReportResp, error) {
	resp := &pb.StatPvPReportResp{}
	// 名将记录
	{
		general := in.General
		if general.Uid > 0 {
			if err := l.svcCtx.StatisticsPvpModel.TransactCtx(l.ctx, nil, func(tx *gorm.DB) error {
				if err := l.svcCtx.StatisticsPvpModel.UpdateRecord(l.ctx, tx, general.Uid,
					&model.UpdateRecordProps{General: true}); err != nil {
					if errors.Is(err, gormx.ErrRowsAffectedZero) {
						// insert
						if err = l.svcCtx.StatisticsPvpModel.Insert(l.ctx, tx, &model.StatisticsPvp{
							UserId:       general.Uid,
							GeneralCount: 1,
						}); err != nil {
							return errors.Wrapf(err, ErrInsertErr, err)
						}
					} else {
						return errors.Wrapf(err, ErrUpdateErr, err)
					}
				}
				return nil
			}); err != nil {
				l.Logger.Errorf("PvP名将记录事务执行失败, err:%v", err)
			}
		}
	}
	// 玩家记录
	{
		// 获胜记录
		if err := l.reports(true, in.WinItems); err != nil {
			l.Logger.Errorf("获胜PvP记录失败, err:%v", err)
		}
		// 战败记录
		if err := l.reports(false, in.LostItems); err != nil {
			l.Logger.Errorf("战败PvP记录失败, err:%v", err)
		}
	}
	// 积分记录
	{
		battleReportCfg := l.svcCtx.Config.Integral.BattleReport
		// 名将积分
		if in.General.Uid > 0 {
			_, err := l.svcCtx.UserIntegralModel.ChangeIntegral(l.ctx, nil, in.General.Uid, battleReportCfg.GeneralIntegral)
			if err != nil {
				l.Logger.Errorf("名将积分更新失败, err:%v", err)
			}
			resp.General = &pb.StatPvPReportResp_Item{
				Uid:           in.General.Uid,
				Uname:         in.General.Uname,
				AddonIntegral: battleReportCfg.GeneralIntegral,
			}
		}
		winItemResp := make([]*pb.StatPvPReportResp_Item, 0, len(in.WinItems))
		lostItemResp := make([]*pb.StatPvPReportResp_Item, 0, len(in.LostItems))

		if err := l.svcCtx.UserIntegralModel.TransactCtx(l.ctx, nil, func(tx *gorm.DB) error {
			for _, item := range in.WinItems {
				uid, damage := item.Uid, item.Damage
				addIntegral := int64(float64(damage) * battleReportCfg.WinRadio)
				_, err := l.svcCtx.UserIntegralModel.ChangeIntegral(l.ctx, tx, uid, addIntegral)
				if err != nil {
					return err
				}
				winItemResp = append(winItemResp, &pb.StatPvPReportResp_Item{
					Uid:           uid,
					Uname:         item.Uname,
					AddonIntegral: addIntegral,
				})
			}
			for _, item := range in.LostItems {
				uid, damage := item.Uid, item.Damage
				addIntegral := int64(float64(damage) * battleReportCfg.LostRadio)
				_, err := l.svcCtx.UserIntegralModel.ChangeIntegral(l.ctx, tx, uid, addIntegral)
				if err != nil {
					return err
				}
				lostItemResp = append(lostItemResp, &pb.StatPvPReportResp_Item{
					Uid:           uid,
					Uname:         item.Uname,
					AddonIntegral: addIntegral,
				})
			}
			return nil
		}); err != nil {
			l.Logger.Errorf("战局积分计算失败, err:%v", err)
		}
		resp.WinItems = winItemResp
		resp.LostItems = lostItemResp
	}
	l.Logger.Info("PvP战报，记录成功...")
	return resp, nil
}

func (l *StatPvpReportLogic) reports(win bool, items []*pb.StatPvPReportReq_Item) error {
	return l.svcCtx.StatisticsPvpModel.TransactCtx(l.ctx, nil, func(tx *gorm.DB) error {
		for _, item := range items {
			if err := l.svcCtx.StatisticsPvpModel.UpdateRecord(l.ctx, tx, item.Uid,
				&model.UpdateRecordProps{
					Win:             win,
					Lost:            !win,
					Damage:          &item.Damage,
					DeDamage:        &item.DeDamage,
					KillUnitCount:   &item.KillUnit,
					DeKillUnitCount: &item.DeKillUnit,
				}); err != nil {
				if errors.Is(err, gormx.ErrRowsAffectedZero) {
					// insert
					var winCount int64
					var lostCount int64
					if win {
						winCount = 1
					} else {
						lostCount = 1
					}
					if err = l.svcCtx.StatisticsPvpModel.Insert(l.ctx, tx, &model.StatisticsPvp{
						UserId:          item.Uid,
						WinCount:        winCount,
						LostCount:       lostCount,
						Damage:          item.Damage,
						DeDamage:        item.DeDamage,
						KillUnitCount:   item.KillUnit,
						DeKillUnitCount: item.DeKillUnit,
					}); err != nil {
						return errors.Wrapf(err, ErrInsertErr, err)
					}
				} else {
					return errors.Wrapf(err, ErrUpdateErr, err)
				}
			}
		}
		return nil
	})
}
